%******************************************************************************;
%* SAS macro Chk4MR   vs. 0.0.0 , 20 Jun 2001, by Jim Groeneveld, Netherlands *;
%******************************************************************************;
%*                                                                            *;
   %MACRO   Chk4MR     /* Check For Multiple Records within IdList in Source  */
/* -------------------------------------------------------------------------- */
/* Argument Default       Description and remarks      Required/Optional: R/O */
/* -------- --------      --------------------------------------------------- */
(  SourceDs=_LAST_     /* Source data set specification                     R */
,  IdList  =           /* Variable List of which multiple Identical value   R */
                       /*     combinations determine a single new record      */
                       /*     (often variables like a Case Number)            */
,  Check   =           /* Name of global macro variable with check result=  R */
                       /*    OK:    multiple Records found within IdList      */
                       /*    NO: no multiple Records found within IdList      */
,  ForMyUse=           /* argument for the author's use only, do not use it ! */
)/ DES     =             'Multiple Partial Records-Single Records' /* STORE */ ;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* File       : Chk4MR.SAS / Chk4MR.000 ( / Chk4MR__.SAS )                    *;
%* Directory  : .................                                             *;
%*                                                                            *;
%* Purpose    : Check For Multiple Records within IdList in Source            *;
%*                                                                            *;
%* Programmer : Jim Groeneveld                                                *;
%* Date       : 20 Jun 2001                                                   *;
%* Version    : 0.00                                                          *;
%*                                                                            *;
%* SAS version: 6.12 under Windows 95                                         *;
%*                                                                            *;
%* Validation : -                                                             *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* Notes:                                                                     *;
%* ======                                                                     *;
%*                                                                            *;
%* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*;
%*                                                                            *;
%* History of Chk4MR:                                                         *;
%* ==================                                                         *;
%* version  date         description of changes, additions, improvements      *;
%* -------  -----------  ---------------------------------------------------- *;
%* 0.0      20 Jun 2001  1st working release                                  *;
%*                                                                            *;
%******************************************************************************;

%* OPTIONS MPRINT MERROR SERROR MLOGIC SYMBOLGEN MACROGEN;
%* remove comment chars for testing ;

  %IF ( %UPCASE(&ForMyUse) = COMPILE ) %THEN
  %DO;
    %ValueLst (ForMyUse=Compile);
    %GOTO Finish;
  %END;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* LOCAL macro variables declarations;
%*___________________________________;

 %LOCAL    /* Define internal macro variables explicitely local */
  MacName  /* This macros external name: Chk4MR */
  I        /* element index */
  Scratch  /* stand-in name for Scratch datasets XX_Scrat */
  OldCount /* Count variable of previous Proc Freq */
  ErrCount /* number of erroneously specified arguments */
  Err      /* text 'E R R O R' as consecutive characters */
  Warn     /* text 'W A R N I N G' as consecutive characters */
  ;

%* Stand-in variable and dataset names;
  %LET MacName  = Chk4MR;
  %LET Scratch  = XX_Scrat;
  %LET OldCount = XX_Count;
  %LET Err      = %SYSFUNC(COMPRESS(E R R O R, %STR( )));
  %LET Warn     = %SYSFUNC(COMPRESS(W A R N I N G, %STR( )));

%* Macro header in LOG file;
  %PUT;
  %PUT |~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~|;
  %PUT |   Macro &MacName, vs. 0.0 by Jim Groeneveld, 20 Jun 2001   |;
  %PUT |    Check For Multiple Records within IdList in Source    |;
  %PUT |__________________________________________________________|;
  %PUT;

%* Feedback of specified arguments;
  %PUT &MacName: SourceDs = &SourceDs;
  %PUT &MacName: IdList   = &IdList;
  %PUT &MacName: Check    = &Check;
  %*PUT &MacName: ForMyUse = &ForMyUse;

*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
* Check arguments for legal contents and discrepancies, calling e_r_r_o_r_s    ;
*______________________________________________________________________________;

  %LET ErrCount = 0;
  %IF ( &SourceDs EQ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: SourceDs argument not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF ( %SYSFUNC(EXIST(&SourceDs)) EQ 0 ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Dataset &SourceDs does not exist,;
    %PUT                       macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF ( &IdList EQ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: IdList argument not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE %IF ( &Check EQ ) %THEN
  %DO;
    %PUT *** &MacName *** &ERR: Check argument not specified, macro &MacName will abort;
    %LET ErrCount = %EVAL ( &ErrCount + 1);
  %END;
  %ELSE
  %DO; %* Check for non-existing variables within dataset;
   %IF ( %UPCASE(&IdList) NE ) %THEN
    %ChkVar (Data=&SourceDs, VarList=&IdList, CalledBy=&MacName,
             Result = ErrCount); %* adding up non-existent variables ;
  %END;
  %IF ( &ErrCount GT 0 ) %THEN %GOTO Interupt;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Macro actually starts here ;
%*____________________________;

%*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~;
%* Determine whether each IdList combination occurs only exactly once: NOMR ;
%*__________________________________________________________________________;

%* Make IdList with multiplication (*) characters in between;
  %LET IdFrqLst = %SCAN(&IdList,1,%STR( ));
  %LET I = 2;
  %LET Id = %SCAN(&IdList,&I,%STR( ));
  %DO %WHILE (&Id NE);
    %LET IdFrqLst = &IdFrqLst * &Id;
    %LET I = %EVAL( &I + 1 );
    %LET Id = %SCAN(&IdList,&I,%STR( ));
  %END; %*=== While;

  PROC FREQ DATA=&SourceDs (KEEP=&IdList);
    TABLE &IdFrqLst / NOPRINT NOPERCENT OUT=&Scratch; RUN;
  PROC FREQ DATA=&Scratch  (KEEP=Count RENAME=(Count=&OldCount));
    TABLE &OldCount / NOPRINT NOPERCENT OUT=&Scratch; RUN;

  DATA _NULL_;
    SET &Scratch (KEEP=&OldCount) NOBS=Nobs;
    IF (Nobs EQ 1 AND &OldCount EQ 1) THEN
    DO;
      CALL EXECUTE ('%LET' || " &&Check = NO;");
      PUT "&Check=NO  Multiple Records found for each &IdList combination";
    END;
    ELSE
    DO;
      CALL EXECUTE ('%LET' || " &&Check = OK;");
      PUT "&Check=OK, Multiple Records found for each &IdList combination";
    END;
    STOP;
  RUN;


%Aborting:

PROC DATASETS NOLIST;
  DELETE &Scratch;   %* Delete scratch datasets;
RUN;

%Interupt:

 %IF ( &ErrCount GT 0 ) %THEN
 %DO;
   DATA _NULL_;
     ABORT /*ABEND*/;
   RUN;
 %END;

%Finish:

%***********;
%MEND Chk4MR; * --------------------end-of-macro-Chk4MR------------------------;
************;

